﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Configuration;
using System.Drawing.Imaging;


[ToolboxBitmap(typeof(TrackBar))]
[DefaultEvent("Scroll"), DefaultProperty("BarInnerColor")]
public partial class MyTrackBar : Control
{
    // ========================================================================
    //  Events
    // ========================================================================
    [Description("Event fires when the Value property changes")]
    [Category("Action")]
    public event EventHandler ValueChanged;

    [Description("Event fires when the Slider position is changed")]
    [Category("Behavior")]
    //public event ScrollEventHandler Scroll;
    public event System.EventHandler Scroll;

    // ========================================================================
    //  Properties
    // ========================================================================
    private Rectangle _thumbRect; //bounding rectangle of thumb area
    private Rectangle _barRect; //bounding rectangle of bar area
    private Rectangle rect_HalfBar;
    private Rectangle _thumbHalfRect;

    // ========================================================================
    //  Properties
    // ========================================================================
    private int _thumbSize = 15;
    [Description("Set slider-thumb size")]
    [Category("Slider")]
    [DefaultValue(15)]
    public int ThumbSize
    {
        get { return _thumbSize; }
        set
        {
            if (value > 0 &
                value < (_barOrientation == Orientation.Horizontal ? ClientRectangle.Width : ClientRectangle.Height))
                _thumbSize = value;
            else
                throw new ArgumentOutOfRangeException(
                    "TrackSize has to be greather than zero and lower than half of Slider width");
            Invalidate();
        }
    }

    private Size _thumbRoundRectSize = new Size(8, 8);
    [Description("Set slider-thumb round rect size")]
    [Category("Slider")]
    [DefaultValue(typeof(Size), "8; 8")]
    public Size ThumbRoundRectSize
    {
        get { return _thumbRoundRectSize; }
        set
        {
            int h = value.Height, w = value.Width;
            if (h <= 0) h = 1;
            if (w <= 0) w = 1;
            _thumbRoundRectSize = new Size(w, h);
            Invalidate();
        }
    }

    private Size _borderRoundRectSize = new Size(8, 8);
    [Description("Set external border round rect size")]
    [Category("Slider")]
    [DefaultValue(typeof(Size), "8; 8")]
    public Size BorderRoundRectSize
    {
        get { return _borderRoundRectSize; }
        set
        {
            int h = value.Height, w = value.Width;
            if (h <= 0) h = 1;
            if (w <= 0) w = 1;
            _borderRoundRectSize = new Size(w, h);
            Invalidate();
        }
    }

    private Orientation _barOrientation = Orientation.Horizontal;
    [Description("Set orientation")]
    [Category("Slider")]
    [DefaultValue(Orientation.Horizontal)]
    public Orientation Orientation
    {
        get { return _barOrientation; }
        set
        {
            _barOrientation = value;
            int temp = Width;
            Width = Height;
            Height = temp;
            Invalidate();
        }
    }

    private int _tickCount = 10;
    [Description("Set the number of Ticks")]
    [Category("")]
    [DefaultValue(10)]
    public int TickCount
    {
        get { return _tickCount; }
        set
        {
            _tickCount = value;
            Invalidate();
        }
    }

    private System.Windows.Forms.TickStyle _tickStyle = System.Windows.Forms.TickStyle.TopLeft;
    [Description("Set Tick Style")]
    [Category("")]
    [DefaultValue(System.Windows.Forms.TickStyle.TopLeft)]
    public System.Windows.Forms.TickStyle TickStyle
    {
        get { return _tickStyle; }
        set
        {
            _tickStyle = value;
            Invalidate();
        }
    }

    private float _logaritmicity = 1;
    [Description("Logaritmicity (1 = linear)")]
    [Category("")]
    [DefaultValue(1)]
    public float Logaritmicity
    {
        get { return _logaritmicity; }
        set
        {
            if (value < 1) value = 1;
            _logaritmicity = value;
            if (ValueChanged != null) ValueChanged(this, new EventArgs());
            Invalidate();
        }
    }

    private int _smartResolutionFigures = 0;
    [Description("0 = disabled / 1 to 9 = number of significant figures")]
    [Category("")]
    [DefaultValue(0)]
    public int SmartResolutionFigures
    {
        get { return _smartResolutionFigures; }
        set
        {
            if (value < 0) value = 0;
            if (value > 9) value = 9;
            _smartResolutionFigures = value;
            if (ValueChanged != null) ValueChanged(this, new EventArgs());
            Invalidate();
        }
    }

    private int _value = 0;
    [Description("Set the slider value")]
    [Category("")]
    [DefaultValue(50)]
    public int Value
    {
        get 
        {
            double v = _value; 
            if (_logaritmicity > 1)
            {
                v = (v - _barMinimum) / (double)(_barMaximum - _barMinimum);
                v = Math.Pow(v, _logaritmicity);
                v = v * (_barMaximum - _barMinimum) + _barMinimum;
                v = (int)(v + 0.5);
            }
            if (_smartResolutionFigures > 0)
            {
                v = SmartResolution(v, _smartResolutionFigures);
            }
            return (int)(v);
        }
        set
        {
            double v = value;
            if (_logaritmicity > 1)
            {
                v = v + 0.5;
                v = (v - _barMinimum) / (double)(_barMaximum - _barMinimum);
                v = Math.Pow(v, 1 / _logaritmicity);
                v = v * (_barMaximum - _barMinimum) + _barMinimum;
                v = (int)(v + 0.5);
            }
            if (v < _barMinimum) v = _barMinimum;
            if (v > _barMaximum) v = _barMaximum;
            _value = (int)v;
            if (ValueChanged != null) ValueChanged(this, new EventArgs());
            Invalidate();
        }
    }

    private int _barMinimum = 0;
    [Description("Set the minimal value")]
    [Category("")]
    [DefaultValue(0)]
    public int Minimum
    {
        get { return _barMinimum; }
        set
        {
            // Calling the property "Value" the value is limited to MinMax 
            // and is corrected for logarithmic bars
            // and the event "ValueChanged" is called
            // and the function "Invalidate()" is called
            int oldValue = Value;
            _barMinimum = value;
            Value = oldValue;
        }
    }

    private int _barMaximum = 100;
    [Description("Set the maximal value")]
    [Category("")]
    [DefaultValue(100)]
    public int Maximum
    {
        get { return _barMaximum; }
        set
        {
            // Calling the property "Value" the value is limited to MinMax 
            // and is corrected for logarithmic bars
            // and the event "ValueChanged" is called
            // and the function "Invalidate()" is called
            int oldValue = Value;
            _barMaximum = value;
            Value = oldValue;
        }
    }

    private int _defaultValue = 50;
    [Description("Set the default value (for mouse right button)")]
    [Category("")]
    [DefaultValue(50)]
    public int DefaultValue
    {
        get { return _defaultValue; }
        set
        {
            _defaultValue = value;
        }
    }

    private uint _smallChange = 1;
    [Description("Set trackbar small change")]
    [Category("")]
    [DefaultValue(1)]
    public uint SmallChange
    {
        get { return _smallChange; }
        set { _smallChange = value; }
    }

    private uint _largeChange = 5;
    [Description("Set trackbar large change")]
    [Category("")]
    [DefaultValue(5)]
    public uint LargeChange
    {
        get { return _largeChange; }
        set { _largeChange = value; }
    }

    private bool _mouseEffects = true;
    [Description("Set whether mouse entry and exit actions have impact on how control look")]
    [Category("Behavior")]
    [DefaultValue(true)]
    public bool MouseEffects
    {
        get { return _mouseEffects; }
        set
        {
            _mouseEffects = value;
            Invalidate();
        }
    }

    private int _mouseWheelSpeed = 1;
    [Description("Set to 1 for slowest speed. Set to 0 to disable mouse wheel.")]
    [Category("")]
    [DefaultValue(1)]
    public int MouseWheelSpeed
    {
        get { return _mouseWheelSpeed; }
        set
        {
            _mouseWheelSpeed = value;
        }
    }

    private bool _showTextAlways = false;
    [Description("True to show the text also when mouse not hovering")]
    [Category("Behavior")]
    [DefaultValue(false)]
    public bool ShowTextAlways
    {
        get { return _showTextAlways; }
        set
        {
            _showTextAlways = value;
            Invalidate();
        }
    }

    private Color _textColor = Color.DarkBlue;
    Brush _brushTextColor = new SolidBrush(Color.DarkBlue);
    [Description("Set the text color - Transparent to hide it")]
    [Category("Colors")]
    [DefaultValue(typeof(Color), "Silver")]
    public Color TextColor
    {
        get { return _textColor; }
        set
        {
            _textColor = value;
            _brushTextColor = new SolidBrush(_textColor);
            Invalidate();
        }
    }

    private Color __roundRectBorderColor = Color.Gray;
    private Pen __roundRectBorderPen = new Pen(Color.FromArgb(200, Color.Gray)); 
    [Description("Set the color of the border - Transparent to hide it")]
    [Category("Colors")]
    [DefaultValue(typeof(Color), "Gray")]
    public Color ColorRectangleBorder
    {
        get { return __roundRectBorderColor; }
        set
        {
            __roundRectBorderColor = value;
            __roundRectBorderPen = new Pen(__roundRectBorderColor);
            __roundRectBorderPen.DashStyle = DashStyle.Dot;
            Invalidate();
        }
    }

    private Color _thumbOuterColor = Color.Silver;
    [Description("Set Slider thumb outer color")]
    [Category("Colors")]
    [DefaultValue(typeof(Color), "Silver")]
    public Color ThumbOuterColor
    {
        get { return _thumbOuterColor; }
        set
        {
            _thumbOuterColor = value;
            Invalidate();
        }
    }

    private Color _thumbInnerColor = Color.WhiteSmoke;
    [Description("Set Slider thumb inner color")]
    [Category("Colors")]
    [DefaultValue(typeof(Color), "WhiteSmoke")]
    public Color ThumbInnerColor
    {
        get { return _thumbInnerColor; }
        set
        {
            _thumbInnerColor = value;
            Invalidate();
        }
    }

    private Color _thumbPenColor = Color.Gray;
    [Description("Set Slider thumb pen color")]
    [Category("Colors")]
    [DefaultValue(typeof(Color), "Gray")]
    public Color ThumbPenColor
    {
        get { return _thumbPenColor; }
        set
        {
            _thumbPenColor = value;
            Invalidate();
        }
    }

    private Color _barOuterColor = Color.LightGray;
    [Description("Set Slider bar outer color")]
    [Category("Colors")]
    [DefaultValue(typeof(Color), "LightGray")]
    public Color BarOuterColor
    {
        get { return _barOuterColor; }
        set
        {
            _barOuterColor = value;
            Invalidate();
        }
    }

    private Color _barInnerColor = Color.WhiteSmoke;
    [Description("Set Slider bar inner color")]
    [Category("Colors")]
    [DefaultValue(typeof(Color), "WhiteSmoke")]
    public Color BarInnerColor
    {
        get { return _barInnerColor; }
        set
        {
            _barInnerColor = value;
            Invalidate();
        }
    }

    private Color _barPenColor = Color.Transparent;
    [Description("Set Slider bar pen color")]
    [Category("Colors")]
    [DefaultValue(typeof(Color), "Transparent")]
    public Color BarPenColor
    {
        get { return _barPenColor; }
        set
        {
            _barPenColor = value;
            Invalidate();
        }
    }


    // ========================================================================
    // Constructors
    // ========================================================================
    public MyTrackBar(int min, int max, int value)
    {
        //InitializeComponent();
        SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.OptimizedDoubleBuffer |
                 ControlStyles.ResizeRedraw | ControlStyles.Selectable |
                 ControlStyles.SupportsTransparentBackColor | ControlStyles.UserMouse |
                 ControlStyles.UserPaint, true);
        BackColor = Color.Transparent;

        Minimum = min;
        Maximum = max;
        Value = value;
    }

    // Initializes a new instance of the <see cref="MyTrackBar"/> class.
    public MyTrackBar() : this(0, 100, 50) {}
  

    // ========================================================================
    //  Overrided events
    // ========================================================================
    private bool _mouseInRegion = false;
    protected override void OnEnabledChanged(EventArgs e)
    {
        base.OnEnabledChanged(e);
        Invalidate();
    }

    protected override void OnMouseEnter(EventArgs e)
    {
        base.OnMouseEnter(e);
        _mouseInRegion = true;
        Invalidate();
    }

    protected override void OnMouseLeave(EventArgs e)
    {
        base.OnMouseLeave(e);
        _mouseInRegion = false;
        _mouseInThumbRegion = false;
        Invalidate();
    }

    float deltaX;
    float deltaY;
    protected override void OnMouseDown(MouseEventArgs e)
    {
        base.OnMouseDown(e);
        if (e.Button == MouseButtons.Left)
        {
            deltaX = e.Location.X - (_thumbRect.Left + _thumbRect.Right) / 2 -1;
            deltaY = e.Location.Y - (_thumbRect.Bottom + _thumbRect.Top) / 2 -1;
            Capture = true;
            if (Scroll != null) Scroll(this, new ScrollEventArgs(ScrollEventType.ThumbTrack, Value));
            if (ValueChanged != null) ValueChanged(this, new EventArgs());
            OnMouseMove(e);
        }
        if (e.Button == MouseButtons.Right)
        {
            Value = _defaultValue;
        }
    }

    private bool _mouseInThumbRegion = false;
    protected override void OnMouseMove(MouseEventArgs e)
    {
        base.OnMouseMove(e);
        _mouseInThumbRegion = IsPointInRect(e.Location, _thumbRect);
        if (Capture & e.Button == MouseButtons.Left)
        {
            ScrollEventType set = ScrollEventType.ThumbPosition;
            int margin = _thumbSize >> 1;
            if (_barOrientation == Orientation.Horizontal)
            {
                int p = e.Location.X - margin - (int)deltaX;
                deltaX *= 0.8f;
                float coef = (float)(_barMaximum - _barMinimum) /
                         (float) (ClientSize.Width  - 2 * margin);
                _value = (int)(p * coef + _barMinimum + 0.5);
            }
            else
            {
                int p = ClientSize.Height - (e.Location.Y + margin - (int)deltaY);
                deltaY *= 0.8f;
                float coef = (float)(_barMaximum - _barMinimum) /
                         (float) (ClientSize.Height - 2 * margin);
                _value = (int)(p * coef + _barMinimum + 0.5);
            }

            if (_value <= _barMinimum)
            {
                _value = _barMinimum;
                set = ScrollEventType.First;
            }
            else if (_value >= _barMaximum)
            {
                _value = _barMaximum;
                set = ScrollEventType.Last;
            }

            if (Scroll != null) Scroll(this, new ScrollEventArgs(set, Value));
            if (ValueChanged != null) ValueChanged(this, new EventArgs());
        }
        Invalidate();
    }

    protected override void OnMouseUp(MouseEventArgs e)
    {
        base.OnMouseUp(e);
        Capture = false;
        _mouseInThumbRegion = IsPointInRect(e.Location, _thumbRect);
        if (Scroll != null) Scroll(this, new ScrollEventArgs(ScrollEventType.EndScroll, Value));
        if (ValueChanged != null) ValueChanged(this, new EventArgs());
        Invalidate();
    }

    protected override void OnMouseWheel(MouseEventArgs e)
    {
        base.OnMouseWheel(e);
        if (_mouseWheelSpeed == 0) return;
        float v;
        v = _mouseWheelSpeed * e.Delta / 120f;
        if (v > 0 && v < 1) v = 1;
        if (v < 0 && v > -1) v = -1;
        OffsetValueBy(Convert.ToInt32(v));
        this.Focus();
    }

    protected override void OnGotFocus(EventArgs e)
    {
        base.OnGotFocus(e);
        Invalidate();
    }

    protected override void OnLostFocus(EventArgs e)
    {
        base.OnLostFocus(e);
        Invalidate();
    }

    protected override void OnKeyDown(KeyEventArgs e)
    {
        base.OnKeyDown(e);
        switch (e.KeyCode)
        {
            case Keys.Down:
            case Keys.Left:
                OffsetValueBy(-(int)_smallChange);
                if (Scroll != null) Scroll(this, new ScrollEventArgs(ScrollEventType.SmallDecrement, Value));
                break;
            case Keys.Up:
            case Keys.Right:
                OffsetValueBy((int)_smallChange);
                if (Scroll != null) Scroll(this, new ScrollEventArgs(ScrollEventType.SmallIncrement, Value));
                break;
            case Keys.Home:
                Value = _barMinimum;
                break;
            case Keys.End:
                Value = _barMaximum;
                break;
            case Keys.PageDown:
                OffsetValueBy(-(int)_largeChange);
                if (Scroll != null) Scroll(this, new ScrollEventArgs(ScrollEventType.LargeDecrement, Value));
                break;
            case Keys.PageUp:
                OffsetValueBy(+(int)_largeChange);
                if (Scroll != null) Scroll(this, new ScrollEventArgs(ScrollEventType.LargeIncrement, Value));
                break;
        }
        if (Scroll != null && Value == _barMinimum) Scroll(this, new ScrollEventArgs(ScrollEventType.First, Value));
        if (Scroll != null && Value == _barMaximum) Scroll(this, new ScrollEventArgs(ScrollEventType.Last, Value));
        Point pt = PointToClient(Cursor.Position);
        OnMouseMove(new MouseEventArgs(MouseButtons.None, 0, pt.X, pt.Y, 0));
    }


    protected override bool ProcessDialogKey(Keys keyData)
    {
        if (keyData == Keys.Tab | ModifierKeys == Keys.Shift)
            return base.ProcessDialogKey(keyData);
        else
        {
            OnKeyDown(new KeyEventArgs(keyData));
            return true;
        }
    }

    // ========================================================================
    //  Help functions
    // ========================================================================
    public static GraphicsPath CreateRoundRectPath(Rectangle rect, Size size)
    {
        GraphicsPath gp = new GraphicsPath();
        gp.AddLine(rect.Left + size.Width / 2, rect.Top, rect.Right - size.Width / 2, rect.Top);
        gp.AddArc(rect.Right - size.Width, rect.Top, size.Width, size.Height, 270, 90);

        gp.AddLine(rect.Right, rect.Top + size.Height / 2, rect.Right, rect.Bottom - size.Width / 2);
        gp.AddArc(rect.Right - size.Width, rect.Bottom - size.Height, size.Width, size.Height, 0, 90);

        gp.AddLine(rect.Right - size.Width / 2, rect.Bottom, rect.Left + size.Width / 2, rect.Bottom);
        gp.AddArc(rect.Left, rect.Bottom - size.Height, size.Width, size.Height, 90, 90);

        gp.AddLine(rect.Left, rect.Bottom - size.Height / 2, rect.Left, rect.Top + size.Height / 2);
        gp.AddArc(rect.Left, rect.Top, size.Width, size.Height, 180, 90);
        return gp;
    }

    public static Color[] TransparentColors(int alpha, params Color[] colors)
    {
        Color[] colorsToReturn = new Color[colors.Length];
        for (int i = 0; i < colors.Length; i++)
        {
            colorsToReturn[i] = Color.FromArgb(alpha, 
                                               colors[i].R,
                                               colors[i].G,
                                               colors[i].B);
        }
        return colorsToReturn;
    }

    public static Color[] DesaturateColors(params Color[] colors)
    {
        Color[] colorsToReturn = new Color[colors.Length];
        for (int i = 0; i < colors.Length; i++)
        {
            int gray = (int)(colors[i].R * 0.3 + 
                             colors[i].G * 0.6 + 
                             colors[i].B * 0.1);
            colorsToReturn[i] = Color.FromArgb(-0x010101 * (255 - gray) - 1);
        }
        return colorsToReturn;
    }

    public static Color[] LightenColors(params Color[] colors)
    {
        Color[] colorsToReturn = new Color[colors.Length];
        for (int i = 0; i < colors.Length; i++)
        {
            colorsToReturn[i] = ControlPaint.Light(colors[i], 0.1f);
        }
        return colorsToReturn;
    }

    public static Color[] DarkenColors(params Color[] colors)
    {
        Color[] colorsToReturn = new Color[colors.Length];
        for (int i = 0; i < colors.Length; i++)
        {
            colorsToReturn[i] = ControlPaint.Dark(colors[i], 0);
        }
        return colorsToReturn;
    }

    private void OffsetValueBy(int offset)
    {
        int v = _value + offset;
        if (v < _barMinimum) v = _barMinimum;
        if (v > _barMaximum) v = _barMaximum;
        _value = v;
        if (ValueChanged != null) ValueChanged(this, new EventArgs());
        Invalidate();
    }

    private static bool IsPointInRect(Point pt, Rectangle rect)
    {
        if (pt.X > rect.Left & pt.X < rect.Right &
            pt.Y > rect.Top & pt.Y < rect.Bottom) return true;
        else return false;
    }

    private int SmartResolution(double n, int precision)
    {
        int order = (int)Math.Log10(n) - precision + 1;
        if (order < 1) return (int)n;
        order = (int)Math.Pow(10, order);
        return order * (int)(n / order);
    }


    // ========================================================================
    //  Paint
    // ========================================================================
    protected override void OnPaint(PaintEventArgs e)
    {
        if (Enabled)
        {
            if (_mouseEffects && _mouseInRegion)
            {
                Color[] lightenedColors = LightenColors(_thumbOuterColor, _thumbInnerColor,
                                                        _thumbPenColor, _barOuterColor,
                                                        _barInnerColor, _barPenColor);
                DrawSlider(e, lightenedColors[0], lightenedColors[1], lightenedColors[2],
                              lightenedColors[3], lightenedColors[4], lightenedColors[5]);
            }
            else
            {
                DrawSlider(e, _thumbOuterColor, _thumbInnerColor, _thumbPenColor,
                              _barOuterColor, _barInnerColor, _barPenColor);
            }
        }
        else
        {
            Color[] disabledColors = TransparentColors(100, 
                                                       _thumbOuterColor, 
                                                       _thumbInnerColor,
                                                       _thumbPenColor,
                                                       _barOuterColor,
                                                       _barInnerColor,
                                                       _barPenColor);

            DrawSlider(e, disabledColors[0], disabledColors[1], disabledColors[2],
                          disabledColors[3], disabledColors[4], disabledColors[5]);
        }
    }


    private void DrawSlider(PaintEventArgs e, Color thumbOuterColorPaint, 
                            Color thumbInnerColorPaint, Color thumbPenColorPaint, 
                            Color barOuterColorPaint, Color barInnerColorPaint,
                            Color barPenColorPaint)
    {
        if (_barMaximum == _barMinimum) return;
        // ---------------------------------------------------------------- set up thumbRect aproprietly
        int thumbPosition = 1;
        if (_tickStyle == TickStyle.TopLeft) thumbPosition = 2;
        if (_tickStyle == TickStyle.BottomRight) thumbPosition = 0;
        if (_barOrientation == Orientation.Horizontal)
        {
            int TrackX = (_value - _barMinimum) * 
                         (ClientRectangle.Width - _thumbSize) / 
                         (_barMaximum - _barMinimum);
            _thumbRect = new Rectangle(TrackX, thumbPosition, 
                                       _thumbSize - 1, ClientRectangle.Height - 3);
        }
        else
        {
            int TrackY = ClientRectangle.Height - _thumbSize - 
                         (_value - _barMinimum) * 
                         (ClientRectangle.Height - _thumbSize) / 
                         (_barMaximum - _barMinimum);
            _thumbRect = new Rectangle(thumbPosition, TrackY, 
                                       ClientRectangle.Width - 3, _thumbSize - 1);
        }

        // ----------------------------------------------------------------- adjust drawing rects
        _barRect = ClientRectangle;
        _thumbHalfRect = _thumbRect;
        LinearGradientMode gradientOrientation;
        if (_barOrientation == Orientation.Horizontal)
        {
            _barRect.Inflate(-6, (int)(-_barRect.Height / 2.5));
            if (_tickStyle == TickStyle.TopLeft) _barRect.Offset(0, 1);
            if (_tickStyle == TickStyle.BottomRight) _barRect.Offset(0, -1);
            rect_HalfBar = _barRect;
            rect_HalfBar.Height /= 2;
            gradientOrientation = LinearGradientMode.Vertical;
            _thumbHalfRect.Height /= 2;
        }
        else
        {
            _barRect.Inflate((int)(-_barRect.Width / 2.5), -6);
            if (_tickStyle == TickStyle.TopLeft) _barRect.Offset(1, 0);
            if (_tickStyle == TickStyle.BottomRight) _barRect.Offset(-1, 0);
            rect_HalfBar = _barRect;
            rect_HalfBar.Width /= 2;
            gradientOrientation = LinearGradientMode.Horizontal;
            _thumbHalfRect.Width /= 2;
        }
        // ----------------------------------------------------------------------- get thumb shape path 
        GraphicsPath thumbPath;
        thumbPath = CreateRoundRectPath(_thumbRect, _thumbRoundRectSize);

        // ----------------------------------------------------------------------- draw bar
        using (LinearGradientBrush lgbBar = new LinearGradientBrush(rect_HalfBar,
                                                                    barOuterColorPaint,
                                                                    barInnerColorPaint,
                                                                    gradientOrientation))
        {
            // ----------------------------------------------------------- draw bar
            lgbBar.WrapMode = WrapMode.TileFlipXY;
            e.Graphics.FillRectangle(lgbBar, _barRect);
      
            // ----------------------------------------------------------- draw bar hilight                  
            using (Pen barPen = new Pen(barPenColorPaint, 0.5f))
            {
                e.Graphics.DrawRectangle(barPen, _barRect);
            }

            // ----------------------------------------------------------- draw ticks
            if (_tickStyle != TickStyle.None)
            {
                Pen tickPen = new Pen(ForeColor);
                if (_barOrientation == Orientation.Horizontal)
                {
                    for (int n = 0; n <= _tickCount; n++)
                    {
                        float x = 3 + _thumbSize / 2f + n * (_barRect.Width - 6) / (float)_tickCount;
                        if (_tickStyle == TickStyle.TopLeft || _tickStyle == TickStyle.Both) 
                            e.Graphics.DrawLine(tickPen, x, 1, x, 6);
                        if (_tickStyle == TickStyle.BottomRight || _tickStyle == TickStyle.Both)
                            e.Graphics.DrawLine(tickPen, x, Height - 2, x, Height - 7);
                    }
                }
                else
                {
                    for (int n = 0; n <= _tickCount; n++)
                    {
                        float y = 3 + _thumbSize / 2f + n * (_barRect.Height - 6) / (float)_tickCount;
                        if (_tickStyle == TickStyle.TopLeft || _tickStyle == TickStyle.Both)
                            e.Graphics.DrawLine(tickPen, 1, y, 6, y);
                        if (_tickStyle == TickStyle.BottomRight || _tickStyle == TickStyle.Both)
                            e.Graphics.DrawLine(tickPen, Width - 2, y, Width - 7, y);
                    }
                }
            }
        }

        // ------------------------------------------------------ draw value as text
        if ((_showTextAlways || _mouseInRegion) && _textColor.A != 0 && Enabled)
        {
            string s = Value.ToString();
           
            if (_barOrientation == Orientation.Horizontal)
            {
                int textWidth = (int)e.Graphics.MeasureString(s, Font).Width;
                int x = _thumbRect.Right + 3;
                if (_thumbRect.Right > ClientSize.Width - textWidth -2) x = _thumbRect.Left - textWidth - 2;
                e.Graphics.DrawString(s, this.Font, _brushTextColor, x, -2);
            }
            else
            {
                int y = _thumbRect.Top - this.Font.Height - 2;
                if (_thumbRect.Top < this.Font.Height) y = _thumbRect.Bottom + 4;
                e.Graphics.DrawString(s, this.Font, _brushTextColor, -1, y);
            }
        }

        // ------------------------------------------------------ draw title text
        if ((_showTextAlways || _mouseInRegion) && 
            _textColor.A != 0 && Text != "" && Enabled)
        {
            int textWidth = (int)e.Graphics.MeasureString(Text, Font).Width;

            if (_barOrientation == Orientation.Horizontal)
            {
                int x = ClientSize.Width - textWidth - 2;
                int y = ClientSize.Height - Font.Height + 2;
                if (_thumbRect.Right > ClientSize.Width - textWidth - 2) x = _thumbRect.Left - textWidth - 2;
                if (x < -2) x = -2;//ClientSize.Width - textWidth - 2;
                e.Graphics.DrawString(Text, Font, _brushTextColor, x, y);
            }
            else
            {
                int x = ClientSize.Width - Font.Height + 2;
                int y = textWidth + 2;
                if (_thumbRect.Top < textWidth + 2) y = ClientSize.Height - 2;
                e.Graphics.RotateTransform(-90);
                e.Graphics.DrawString(Text, Font, _brushTextColor, -y, x, StringFormat.GenericDefault);
                e.Graphics.RotateTransform(90);
            }
        }

        // ------------------------------------------------------ draw external rectangle
        if (__roundRectBorderColor.A != 0)
        {
            Rectangle r = ClientRectangle;
            r.Width -= 2;
            r.Height -= 2;
            using (GraphicsPath gpBorder = CreateRoundRectPath(r, _borderRoundRectSize))
            {
                e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
                e.Graphics.DrawPath(__roundRectBorderPen, gpBorder);
            }
        }

        // --------------------------------------------------------------- draw thumb
        Color newthumbOuterColorPaint = thumbOuterColorPaint;
        Color newthumbInnerColorPaint = thumbInnerColorPaint;
        if (Capture)
        {
            newthumbOuterColorPaint = ControlPaint.Light(thumbOuterColorPaint);
        }
        using (LinearGradientBrush lgbThumb = new LinearGradientBrush(_thumbHalfRect,
                                                                      newthumbOuterColorPaint,
                                                                      newthumbInnerColorPaint,
                                                                      gradientOrientation))
        {
            lgbThumb.WrapMode = WrapMode.TileFlipXY;
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            e.Graphics.FillPath(lgbThumb, thumbPath);
            // ----------------------------------------------------------- draw thumb band
            Color newThumbPenColor = thumbPenColorPaint;
            if (_mouseEffects && (Capture || _mouseInThumbRegion))
                newThumbPenColor = ControlPaint.Dark(newThumbPenColor, 0.3f);
            using (Pen thumbPen = new Pen(newThumbPenColor))
            {
                e.Graphics.DrawPath(thumbPen, thumbPath);
            }
        }
    }

}

